/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_yaml.qpp

    yaml Qore module

    Copyright (C) 2010 - 2022 Qore Technologies, s.r.o.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "yaml-module.h"

static QoreStringNode* q_make_yaml(QoreValue data, int64 flags, int64 width, int64 indent, ExceptionSink* xsink) {
    QoreYamlStringWriteHandler str;
    {
        QoreYamlEmitter emitter(str, flags, width, indent, xsink);
        if (*xsink) {
            return nullptr;
        }

        if (emitter.emit(data)) {
            return nullptr;
        }
    }

    return str.take();
}

static QoreHashNode* q_get_yaml_info() {
    QoreHashNode *h = new QoreHashNode(autoTypeInfo);

    h->setKeyValue("version", new QoreStringNode(yaml_get_version_string()), nullptr);

    int major, minor, patch;
    yaml_get_version(&major, &minor, &patch);

    h->setKeyValue("major", major, nullptr);
    h->setKeyValue("minor", minor, nullptr);
    h->setKeyValue("patch", patch, nullptr);

    return h;
}
# 92 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 97 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 123 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 142 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 165 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 181 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 199 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
// hash getYAMLInfo() {}
static QoreValue f_getYAMLInfo(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 208 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
    return q_get_yaml_info();
}

// hash<auto> get_yaml_info() {}
static QoreValue f_get_yaml_info(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 195 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
    return q_get_yaml_info();
}

// string makeYAML(auto data, int flags = Qore::YAML::None, softint width = -1, softint indent = 2) {}
static QoreValue f_makeYAML_VAVivivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue data = get_param_value(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
    int64 width = HARD_QORE_VALUE_INT(args, 2);
    int64 indent = HARD_QORE_VALUE_INT(args, 3);
# 138 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
    return q_make_yaml(data, flags, width, indent, xsink);
}

// string make_yaml(auto data, int flags = Qore::YAML::None, softint width = -1, softint indent = 2) {}
static QoreValue f_make_yaml_VAVivivi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue data = get_param_value(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
    int64 width = HARD_QORE_VALUE_INT(args, 2);
    int64 indent = HARD_QORE_VALUE_INT(args, 3);
# 119 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
    return q_make_yaml(data, flags, width, indent, xsink);
}

// auto parseYAML(string yaml) {}
static QoreValue f_parseYAML_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* yaml = HARD_QORE_VALUE_STRING(args, 0);
# 176 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
    QoreYamlParser parser(*yaml, xsink);
    return parser.parse();
}

// auto parse_yaml(string yaml) {}
static QoreValue f_parse_yaml_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* yaml = HARD_QORE_VALUE_STRING(args, 0);
# 160 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
    QoreYamlParser parser(*yaml, xsink);
    return parser.parse();
}


DLLLOCAL void init_yaml_functions(QoreNamespace& ns) {
    // hash getYAMLInfo() {}
    ns.addBuiltinVariant("getYAMLInfo", (q_func_n_t)f_getYAMLInfo, QCF_CONSTANT|QCF_DEPRECATED, QDOM_DEFAULT, hashTypeInfo);
    // hash<auto> get_yaml_info() {}
    ns.addBuiltinVariant("get_yaml_info", (q_func_n_t)f_get_yaml_info, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);
    // string makeYAML(auto data, int flags = Qore::YAML::None, softint width = -1, softint indent = 2) {}
    ns.addBuiltinVariant("makeYAML", (q_func_n_t)f_makeYAML_VAVivivi, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 4, autoTypeInfo, QORE_PARAM_NO_ARG, "data", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "flags", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "width", softBigIntTypeInfo, QoreSimpleValue().assign((int64)2), "indent");
    // string make_yaml(auto data, int flags = Qore::YAML::None, softint width = -1, softint indent = 2) {}
    ns.addBuiltinVariant("make_yaml", (q_func_n_t)f_make_yaml_VAVivivi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, autoTypeInfo, QORE_PARAM_NO_ARG, "data", bigIntTypeInfo, QoreSimpleValue().assign((int64)0), "flags", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "width", softBigIntTypeInfo, QoreSimpleValue().assign((int64)2), "indent");
    // auto parseYAML(string yaml) {}
    ns.addBuiltinVariant("parseYAML", (q_func_n_t)f_parseYAML_Vs, QCF_RET_VALUE_ONLY|QCF_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "yaml");
    // auto parse_yaml(string yaml) {}
    ns.addBuiltinVariant("parse_yaml", (q_func_n_t)f_parse_yaml_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "yaml");

}

DLLLOCAL void init_yaml_constants(QoreNamespace& ns) {
# 58 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 65 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 68 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 71 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 74 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 77 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 80 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
//const Yaml1_0 = QYE_VER_1_0;
# 85 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
//const Yaml1_2 = QYE_VER_1_2;
//YNS.addConstant("Yaml1_2", QYE_VER_1_2);
    ns.addConstant("BlockStyle", ((int64)QYE_BLOCK_STYLE));
    ns.addConstant("Canonical", ((int64)QYE_CANONICAL));
    ns.addConstant("EmitSqlNull", ((int64)QYE_EMIT_SQLNULL));
    ns.addConstant("EscapeUnicode", ((int64)QYE_ESCAPE_UNICODE));
    ns.addConstant("ExplicitEndDoc", ((int64)QYE_EXPLICIT_END_DOC));
    ns.addConstant("ExplicitStartDoc", ((int64)QYE_EXPLICIT_START_DOC));
    ns.addConstant("None", ((int64)QYE_NONE));
    ns.addConstant("Yaml1_1", ((int64)QYE_VER_1_1));
# 92 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 97 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 123 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 142 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 165 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 181 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"
# 199 "/home/abuild/rpmbuild/BUILD/qore-yaml-module-1.0.0-build/qore-yaml-module-1.0.0/src/ql_yaml.qpp"

}
